package com.mortals.xhx.common.formdesign.component;

import cn.hutool.core.util.IdUtil;
import com.google.common.collect.Lists;
import com.mortals.xhx.common.code.ComponentEnum;
import com.mortals.xhx.common.formdesign.ColumnsItem;
import com.mortals.xhx.common.formdesign.DesignComponent;
import com.mortals.xhx.common.formdesign.ListItem;
import com.mortals.xhx.common.formdesign.OptionsItem;
import com.mortals.xhx.common.pdu.gen.component.ComponentCons;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 动态表格组件
 *
 * @author: zxfei
 * @date: 2021/10/26 10:35
 * @description:
 **/
public class DesignDynamicTableComponent extends DesignComponent {

    public DesignDynamicTableComponent(String type) {
        super(type);
    }

    @Override
    public ListItem buildDefaultComponent(ComponentCons cons) {
        ListItem listItem = new ListItem();
        String id = "fd_" + IdUtil.fastSimpleUUID();
        listItem.setId(id);
        listItem.set_id(id);
        listItem.setCompType("dynamicTable");
        listItem.setEle("dynamic-table");
        listItem.setCompName("动态表格");
        listItem.setLayout("dynamicItem");
        listItem.setCompIcon("table");
        listItem.setLabelWidth(0);
        listItem.setShowLabel(false);
        listItem.setConfig(true);
        listItem.setButtonAdd(true);
        listItem.setButtonDel(true);
        listItem.setShowIndex(true);
        listItem.setVisible(true);
        listItem.setMultiCheck(false);
        listItem.setShowSummary(false);
        listItem.setSumText("合计");
        listItem.setSummaryText("元");

        if (!ObjectUtils.isEmpty(cons.getColList())) {
            //动态生成列元素
            List<ColumnsItem> columnsItemList = cons.getColList().stream().map(item -> {
                ColumnsItem columnsItem = new ColumnsItem();
                if (item.getColType().equals(ComponentEnum.INPUT.getValue())) {
                    //创建输入框出初始值
                    columnsItem.setId(item.getName());
                    columnsItem.set_id(item.getName());
                    columnsItem.setCompType("input");
                    columnsItem.setCompName("单行文本");
                    columnsItem.setCompIcon("input");
                    columnsItem.setViewType("text");
                    columnsItem.setEle("input");
                    columnsItem.setConfig(true);
                    columnsItem.setShowLabel(true);
                    columnsItem.setLabel(item.getLabel());
                    columnsItem.setLabelWidth(120);
                    columnsItem.setPlaceholder("请输入" + cons.getLabel());
                    columnsItem.setRequired(false);
                    columnsItem.setMaxLength(50);
                    columnsItem.setGutter(15);
                    columnsItem.setSpan(24);
                    columnsItem.setWidth("100%");
                    columnsItem.setClearable(true);
                    columnsItem.setDisabled(false);
                    columnsItem.setReadonly(false);
                    columnsItem.setStatus("normal");
                    columnsItem.setPrefixIcon("");
                    columnsItem.setValue("");
                    columnsItem.setRules(Lists.newArrayList());
                    columnsItem.setRulesType("default");
                    columnsItem.setAppend("");
                    columnsItem.setPrepend("");
                    columnsItem.setLayout("colItem");
                }
                return columnsItem;
            }).collect(Collectors.toList());
            listItem.setColumns(columnsItemList);
        }

        return listItem;
    }


    @Override
    public String getType() {
        return ComponentEnum.DYNAMIC_TABLE.getValue();
    }
}
